"""Exceptions for AQMAN101 from RadonFTLabs"""


class AqmanError(Exception):
    """Generic Aqman Exception"""
    pass


class AqmanConnectionError(AqmanError):
    """Aqman connection Exception"""
    pass



import attr
import datetime
from typing import List, Dict


@attr.s(auto_attribs=True, frozen=True)
class Device:
    """Object holding the Aqman101 device state."""

    serial_number: str
    dsm101_serial_number: str
    date_time: datetime.datetime
    device_type: int
    firmware_version: str
    upload_type: int
    smartplug_enable: int
    temperature: float
    humidity: int
    radon: int
    co2: int
    pm10: int
    pm2d5: int
    pm1: int
    tvoc: int

    @staticmethod
    def from_dict(data):
        return Device(
            serial_number=data["sn"],
            dsm101_serial_number=data["dsm101_sn"],
            date_time=data["dt"],
            device_type=data["devType"],
            firmware_version=data["version"],
            upload_type=data["uploadType"],
            smartplug_enable=data["plugEnable"],
            temperature=data["temp"],
            humidity=data["humi"],
            radon=data["radon"],
            co2=data["co2"],
            pm10=data["pm10"],
            pm2d5=data["pm2d5"],
            pm1=data["pm1"],
            tvoc=data["tvoc"]
        )


@attr.s(auto_attribs=True, frozen=True)
class UserInfo:
    """Object holding the user information of Aqman101 user"""

    username: str
    password: str
    devices: List[str]
    device_cnt: int

    @staticmethod
    def from_list(username: str, password: str, data: List[Dict]):
        device_list: List[str] = []

        for device in data:
            device_list.append(device['sn'])

        return UserInfo(username, password, device_list, len(device_list))



"""Asynchronous Python client for AQMAN101 from RadonFTLabs"""
import asyncio
import socket
from typing import Any, Optional, List

import aiohttp
import async_timeout
from yarl import URL

from .__version__ import __version__
from .exceptions import AqmanError, AqmanConnectionError
from .models import Device, UserInfo
from .const import BASE_URL


class AqmanUser:
    """Main class for handling connections with an AQMAN101"""

    def __init__(
            self,
            id: str = None,
            password: str = None,
            host: str = BASE_URL,
            request_timeout: int = 10,
            session: aiohttp.ClientSession = None):
        """Initialize the AqmanUser class"""
        self._session = session

        self._id = id
        self._password = password
        self._host = host
        self._request_timeout = request_timeout
        self._token = None

    async def _request(self, uri: str, data: Optional[dict] = None,) -> Any:
        """Handle a request to a Aqman101"""
        url = URL.build(scheme="https", host=self._host, path=f"/{uri}")

        if uri == "login":
            method = "POST"
        else:
            method = "GET"
            headers = {
                "Token": f"{self._token}"
            }

        if self._session is None:
            self._session = aiohttp.ClientSession()

        try:
            with async_timeout.timeout(self._request_timeout):
                if method == "POST":
                    response = await self._session.request(
                        method, str(url), json=data,
                    )
                elif method == "GET":
                    response = await self._session.request(
                        method, str(url), params=data, headers=headers
                    )
                response.raise_for_status()
        except asyncio.TimeoutError as exception:
            raise AqmanConnectionError(
                "Timeout occurred while connecting to Aqman 101"
            ) from exception
        except (
            aiohttp.ClientError,
            aiohttp.ClientResponseError,
            socket.gaierror,
        ) as exception:
            raise AqmanConnectionError(
                "Error occurred while communicating with Aqman 101"
            ) from exception

        content_type = response.headers.get("Content-Type", "")
        if "application/json" not in content_type:
            text = await response.text()
            raise AqmanError(
                "Unexpected response from the Aqman 101",
                {"Content-Type": content_type, "response": text},
            )

        return await response.json()

    async def token(self) -> str:
        """Get the token for current session of Aqman101"""
        data = await self._request("login", data={"id": self._id, "password": self._password})
        web_token = data['tokenWeb']
        self._token = web_token
        return web_token

    async def devices_info(self) -> List[str]:
        """Get the list of serial numbers for current user of Aqman101"""
        if self._token == None:
            await self.token()
        data = await self._request("devices", data={"customerId": self._id})
        return UserInfo.from_list(self._id, self._password, data)

    async def close(self) -> None:
        """Close open client session."""
        if self._session:
            await self._session.close()

    async def __aenter__(self) -> "Aqman":
        """Async enter."""
        return self

    async def __aexit__(self, *exc_info) -> None:
        """Async exit."""
        await self.close()


class AqmanDevice:
    """Class for handling single connection with an AQMAN101"""

    def __init__(
            self,
            id: str = None,
            password: str = None,
            deviceid: str = None,
            host: str = BASE_URL,
            request_timeout: int = 10,
            session: aiohttp.ClientSession = None) -> None:
        """Initialize Connection with AQMAN101"""
        self._session = session

        self._id = id
        self._password = password
        self._deviceid = deviceid
        self._host = host
        self._request_timeout = request_timeout
        self._token = None

    async def _request(self, uri: str, data: Optional[dict] = None,) -> Any:
        """Handle a request to a Aqman101"""
        url = URL.build(scheme="https", host=self._host, path=f"/{uri}")

        if uri == "login":
            method = "POST"
        else:
            method = "GET"
            headers = {
                "Token": f"{self._token}"
            }

        if self._session is None:
            self._session = aiohttp.ClientSession()

        try:
            with async_timeout.timeout(self._request_timeout):
                if method == "POST":
                    response = await self._session.request(
                        method, str(url), json=data,
                    )
                elif method == "GET":
                    response = await self._session.request(
                        method, str(url), params=data, headers=headers
                    )
                response.raise_for_status()
        except asyncio.TimeoutError as exception:
            raise AqmanConnectionError(
                "Timeout occurred while connecting to Aqman 101"
            ) from exception
        except (
            aiohttp.ClientError,
            aiohttp.ClientResponseError,
            socket.gaierror,
        ) as exception:
            raise AqmanConnectionError(
                "Error occurred while communicating with Aqman 101"
            ) from exception

        content_type = response.headers.get("Content-Type", "")
        if "application/json" not in content_type:
            text = await response.text()
            raise AqmanError(
                "Unexpected response from the Aqman 101",
                {"Content-Type": content_type, "response": text},
            )

        return await response.json()

    async def token(self) -> str:
        """Get the token for current session of Aqman101"""
        data = await self._request("login", data={"id": self._id, "password": self._password})
        web_token = data['tokenWeb']
        self._token = web_token
        return web_token

    async def state(self) -> Device:
        """Get the current state of Aqman101"""
        if self._token is None:
            await self.token()
        data = await self._request("devices", data={"sn": self._deviceid})
        return Device.from_dict(data)

    async def close(self) -> None:
        """Close open client session."""
        if self._session:
            await self._session.close()

    async def __aenter__(self) -> "Aqman":
        """Async enter."""
        return self

    async def __aexit__(self, *exc_info) -> None:
        """Async exit."""
        await self.close()



#!/usr/bin/env python

import os
import re
import sys

from setuptools import find_packages, setup


def read(*parts):
    """Read file."""
    filename = os.path.join(os.path.abspath(os.path.dirname(__file__)), *parts)
    sys.stdout.write(filename)
    with open(filename, encoding="utf-8", mode="rt") as fp:
        return fp.read()


def get_version():
    """Get current version from code."""
    regex = r"__version__\s=\s\"(?P<version>[\d\.]+?)\""
    path = ("aqman", "__version__.py")
    return re.search(regex, read(*path)).group("version")


with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="python-aqman",  # Replace with your own username
    version=get_version(),
    author="Hosung Kim",
    author_email="samhskim1@gmail.com",
    description="Asynchronous Python Client for Radon FTLabs AQMAN101",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hkder/python-aqman",
    include_package_data=True,
    install_requires=["aiohttp>=3.0.0", "attrs>=19.0.0", "yarl>=1.0.0"],
    license="MIT license",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    zip_safe=False,
)
